<?php
// check_otp.php أو verify_recovery_code.php
require '../config/db.php';
require '../utils/auth.php';

// تعيين المنطقة الزمنية إلى Asia/Riyadh لأغراض العرض والتسجيل
date_default_timezone_set('Asia/Riyadh');

// تحديد المنطقة الزمنية الصحيحة لخادمك (مثلاً UTC-5)
$server_timezone = new DateTimeZone('America/New_York'); // استبدل 'America/New_York' بالمنطقة الزمنية الفعلية للخادم

// قراءة البيانات
$raw_input = file_get_contents("php://input");
$data = json_decode($raw_input, true);

// تصحيح الأخطاء
error_log("Raw Input: " . $raw_input);
error_log("Decoded Data: " . print_r($data, true));
error_log("PHP Current Time (Asia/Riyadh): " . date('Y-m-d H:i:s') . " (" . date_default_timezone_get() . ")");

$phone_number = isset($data['phone_number']) ? trim($data['phone_number']) : null;
$recovery_code = isset($data['recovery_code']) ? trim($data['recovery_code']) : null;

// التحقق من القيم
if (!$phone_number || !$recovery_code) {
    http_response_code(400);
    echo json_encode(["message" => "رقم الهاتف ورمز الاستعادة مطلوبان"]);
    exit();
}

// البحث عن المستخدم
$stmt = $pdo->prepare("SELECT id FROM users WHERE phone_number = ?");
if (!$stmt->execute([$phone_number])) {
    error_log("Execute failed: " . implode(", ", $stmt->errorInfo()));
    http_response_code(500);
    echo json_encode(["message" => "خطأ في الخادم"]);
    exit();
}
$user = $stmt->fetch(PDO::FETCH_ASSOC);

if (!$user) {
    http_response_code(404);
    echo json_encode(["message" => "المستخدم غير موجود"]);
    exit();
}

// البحث عن رمز الاستعادة
$stmt = $pdo->prepare("SELECT * FROM password_recovery WHERE user_id = ? AND recovery_code = ?");
if (!$stmt->execute([$user['id'], $recovery_code])) {
    error_log("Execute failed: " . implode(", ", $stmt->errorInfo()));
    http_response_code(500);
    echo json_encode(["message" => "خطأ في الخادم"]);
    exit();
}
$recovery = $stmt->fetch(PDO::FETCH_ASSOC);

if (!$recovery) {
    error_log("رمز استعادة غير صحيح للرقم: " . $phone_number);
    http_response_code(400);
    echo json_encode(["message" => "رمز الاستعادة غير صحيح"]);
    exit();
}

// التحقق من صلاحية الرمز (15 دقيقة)
try {
    // تأكد من أن 'created_at' يتم تفسيره بمنطقة زمنية الخادم
    $created_at = new DateTime($recovery['created_at'], $server_timezone);
    $now = new DateTime('now', $server_timezone);
    $interval = $now->getTimestamp() - $created_at->getTimestamp();

    // تسجيل الفارق الزمني
    error_log("Created At (Server Timezone): " . $created_at->format('Y-m-d H:i:s'));
    error_log("Now (Server Timezone): " . $now->format('Y-m-d H:i:s'));
    error_log("Interval (seconds): " . $interval);

    if ($interval > 900) { // 900 ثانية = 15 دقيقة
        // حذف الرمز المنتهي الصلاحية
        $stmt = $pdo->prepare("DELETE FROM password_recovery WHERE id = ?");
        if (!$stmt->execute([$recovery['id']])) {
            error_log("Execute failed: " . implode(", ", $stmt->errorInfo()));
        }

        http_response_code(400);
        echo json_encode(["message" => "رمز الاستعادة منتهي الصلاحية"]);
        exit();
    }
} catch (Exception $e) {
    error_log("Error parsing dates: " . $e->getMessage());
    http_response_code(500);
    echo json_encode(["message" => "خطأ في الخادم"]);
    exit();
}

// الرمز صحيح وصالح، قم بمسحه من قاعدة البيانات
$stmt = $pdo->prepare("DELETE FROM password_recovery WHERE id = ?");
if (!$stmt->execute([$recovery['id']])) {
    error_log("Execute failed: " . implode(", ", $stmt->errorInfo()));
}

// إرجاع رسالة نجاح
echo json_encode(["message" => "رمز الاستعادة صالح"]);
?>
