<?php
require '../config/db.php';

// تحليل البيانات الواردة بصيغة JSON
$input = file_get_contents("php://input");
$data = json_decode($input, true);

// تسجيل البيانات المستلمة لتسهيل التتبع والتصحيح
error_log("Received data: " . print_r($data, true));
error_log("Raw input: " . $input);

// استخراج البيانات
$ref_code = $data['ref_code'] ?? null;
$user_id = $data['user_id'] ?? null;

// التحقق من وجود البيانات الضرورية
if (!$ref_code || !$user_id) {
    http_response_code(400);
    echo json_encode(["message" => "رمز المرجع ومعرف المستخدم مطلوبان"]);
    exit();
}

try {
    // استرجاع المعاملة من قاعدة البيانات للتأكد من ارتباطها بالمستخدم
    $stmt = $pdo->prepare("SELECT * FROM transactions WHERE reference = ? AND user_id = ?");
    $stmt->execute([$ref_code, $user_id]);
    $transaction = $stmt->fetch(PDO::FETCH_ASSOC);

    if (!$transaction) {
        http_response_code(404);
        echo json_encode(["message" => "المعاملة غير موجودة أو غير مرتبطة بالمستخدم"]);
        exit();
    }

    // التحقق من حالة المعاملة الحالية
    if (in_array($transaction['status'], ['success', 'completed'])) {
        echo json_encode(["message" => "تم تأكيد المعاملة مسبقًا"]);
        exit();
    }

    // التواصل مع بوابة الدفع لتأكيد المعاملة
    $payment_api_url = "https://gate.sha7nawy.com/api/payment/confirm";
    $headers = [
        "Accept: application/json",
        "Content-Type: application/json", // تأكد من تضمين Content-Type
        "Authorization: iTayeDiG7mqKWtgKRxwcpPertdsyCitxQjqV8oWxzSZvIqbcPjAr2iWv2D8q5rO7pDYyLtcvOKu1vKINYHPoDQO136kOo0SHiLZi" // تأكد من وجود فراغ بعد :
    ];

    $body = [
        "ref_code" => $ref_code  // استخدام 'ref_code' لمطابقة البيانات المطلوبة من بوابة الدفع
    ];

    // تسجيل بيانات الطلب المرسل إلى بوابة الدفع
    error_log("Sending to payment gateway: " . json_encode($body));

    $ch = curl_init($payment_api_url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
    curl_setopt($ch, CURLOPT_POST, true);
    curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($body));
    $response = curl_exec($ch);

    // تسجيل الاستجابة من بوابة الدفع
    error_log("Payment gateway response: " . $response);

    if (curl_errno($ch)) {
        http_response_code(500);
        echo json_encode(["message" => "خطأ في الاتصال ببوابة الدفع: " . curl_error($ch)]);
        curl_close($ch);
        exit();
    }

    $response_data = json_decode($response, true);
    curl_close($ch);

    // التحقق من استجابة بوابة الدفع
    if (isset($response_data['status'])) {
        if ($response_data['status'] === true) {
            // تحقق من حالة المعاملة في البيانات
            if (isset($response_data['data']['status'])) {
                $gateway_status = strtolower(trim($response_data['data']['status']));
                error_log("Gateway status after normalization: " . $gateway_status);

                if (in_array($gateway_status, ['success', 'completed'])) {
                    // بدء المعاملة في قاعدة البيانات
                    $pdo->beginTransaction();

                    // تحديث حالة المعاملة
                    $stmt = $pdo->prepare("UPDATE transactions SET status = ?, last_updated = NOW() WHERE reference = ? AND user_id = ?");
                    $new_status = ($gateway_status === 'completed') ? 'completed' : 'success';
                    $update_status = $stmt->execute([$new_status, $ref_code, $user_id]);
                    $affected_rows = $stmt->rowCount();
                    error_log("Rows updated: " . $affected_rows);

                    if ($update_status && $affected_rows > 0) {
                        // تحديث رصيد المستخدم بالمبلغ المشحون
                        if (isset($response_data['data']['amount'])) {
                            $amount = $response_data['data']['amount'];
                            $stmt = $pdo->prepare("UPDATE users SET balance = balance + ? WHERE id = ?");
                            $update_balance = $stmt->execute([$amount, $user_id]);
                            $affected_rows_balance = $stmt->rowCount();
                            error_log("Rows updated in users balance: " . $affected_rows_balance);

                            if (!$update_balance || $affected_rows_balance < 1) {
                                // فشل تحديث رصيد المستخدم، التراجع
                                $pdo->rollBack();
                                $error_info = $stmt->errorInfo();
                                error_log("Failed to update user balance for user_id: $user_id. Error: " . implode(", ", $error_info));
                                http_response_code(500);
                                echo json_encode(["message" => "فشل تحديث رصيد المستخدم"]);
                                exit();
                            }
                        }

                        // التزام المعاملة
                        $pdo->commit();

                        echo json_encode(["message" => "تم تأكيد المعاملة وتحديث الرصيد"]);
                        exit();
                    } else {
                        // فشل تحديث حالة المعاملة، التراجع
                        $pdo->rollBack();
                        $error_info = $stmt->errorInfo();
                        error_log("Failed to update transaction status to '$new_status' for reference: $ref_code. Error: " . implode(", ", $error_info));
                        http_response_code(500);
                        echo json_encode(["message" => "فشل تحديث حالة المعاملة"]);
                        exit();
                    }
                } else {
                    // حالة المعاملة غير ناجحة، تحديث الحالة إلى 'rejected'
                    $pdo->beginTransaction();

                    $stmt = $pdo->prepare("UPDATE transactions SET status = 'rejected', last_updated = NOW() WHERE reference = ? AND user_id = ?");
                    $update_status = $stmt->execute([$ref_code, $user_id]);
                    $affected_rows_rejected = $stmt->rowCount();
                    error_log("Rows updated to 'rejected': " . $affected_rows_rejected);

                    if ($update_status && $affected_rows_rejected > 0) {
                        $pdo->commit();

                        // الرد برسالة الفشل
                        http_response_code(400);
                        echo json_encode([
                            "message" => $response_data['message'] ?? "فشل تأكيد المعاملة",
                            "data" => $response_data['data'] ?? []
                        ]);
                        exit();
                    } else {
                        $pdo->rollBack();
                        $error_info = $stmt->errorInfo();
                        error_log("Failed to update transaction status to 'rejected' for reference: $ref_code. Error: " . implode(", ", $error_info));
                        http_response_code(500);
                        echo json_encode(["message" => "فشل تحديث حالة المعاملة"]);
                        exit();
                    }
                }
            } else {
                // حالة المعاملة غير محددة في البيانات، تحديث الحالة إلى 'rejected'
                $pdo->beginTransaction();

                $stmt = $pdo->prepare("UPDATE transactions SET status = 'rejected', last_updated = NOW() WHERE reference = ? AND user_id = ?");
                $update_status = $stmt->execute([$ref_code, $user_id]);
                $affected_rows_rejected = $stmt->rowCount();
                error_log("Rows updated to 'rejected' without data.status: " . $affected_rows_rejected);

                if ($update_status && $affected_rows_rejected > 0) {
                    $pdo->commit();

                    // الرد برسالة الفشل
                    http_response_code(400);
                    echo json_encode([
                        "message" => $response_data['message'] ?? "فشل تأكيد المعاملة",
                        "data" => $response_data['data'] ?? []
                    ]);
                    exit();
                } else {
                    $pdo->rollBack();
                    $error_info = $stmt->errorInfo();
                    error_log("Failed to update transaction status to 'rejected' for reference: $ref_code without data.status. Error: " . implode(", ", $error_info));
                    http_response_code(500);
                    echo json_encode(["message" => "فشل تحديث حالة المعاملة"]);
                    exit();
                }
            }
        } else {
            // 'status' غير موجود أو غير صحيح في الاستجابة
            error_log("Invalid or missing 'status' in payment gateway response");
            http_response_code(400);
            echo json_encode([
                "message" => $response_data['message'] ?? "فشل تأكيد المعاملة",
                "data" => $response_data['data'] ?? []
            ]);
            exit();
        }
    } else {
        // 'status' غير موجود في الاستجابة
        error_log("Missing 'status' in payment gateway response");
        http_response_code(400);
        echo json_encode([
            "message" => $response_data['message'] ?? "فشل تأكيد المعاملة",
            "data" => $response_data['data'] ?? []
        ]);
        exit();
    }

} catch (PDOException $e) {
    error_log("PDOException: " . $e->getMessage());
    http_response_code(500);
    echo json_encode(["message" => "Database error: " . $e->getMessage()]);
}
?>
