<?php
require '../config/db.php'; // تأكد من أن هذا الملف يقوم بإعداد اتصالك بقاعدة البيانات بشكل صحيح

// قراءة البيانات الواردة بصيغة JSON من الطلب POST
$data = json_decode(file_get_contents("php://input"), true);

// التحقق من أخطاء تحليل JSON
if (json_last_error() !== JSON_ERROR_NONE) {
    http_response_code(400);
    echo json_encode(["message" => "Invalid JSON: " . json_last_error_msg()]);
    exit();
}

// استخراج البيانات
$user_id = $data['user_id'] ?? null; // تأكد من تمرير user_id في الطلب
$number = $data['number'] ?? null;
$amount = $data['amount'] ?? null;
$method = $data['method'] ?? null;
$client = $data['client'] ?? null;
$details = $data['details'] ?? null;

// التحقق من أن الحقول المطلوبة موجودة
if (empty($user_id) || empty($number) || empty($amount) || empty($method)) {
    http_response_code(400);
    echo json_encode(["message" => "User ID, number, amount, and method are required fields."]);
    exit();
}

// تحديد قائمة بطرق الدفع المقبولة
$validMethods = ['credit_card', 'paypal', 'bank_transfer', 'vfcash'];

// التحقق من صحة طريقة الدفع
if (!in_array($method, $validMethods)) {
    http_response_code(400);
    echo json_encode(["message" => "The selected method is invalid."]);
    exit();
}

// إعداد الطلب للـ API الخارجية
$payment_api_url = "https://gate.sha7nawy.com/api/payment/create";
$headers = [
    "Accept: application/json",
    "Content-Type: application/json",
    "Authorization: iTayeDiG7mqKWtgKRxwcpPertdsyCitxQjqV8oWxzSZvIqbcPjAr2iWv2D8q5rO7pDYyLtcvOKu1vKINYHPoDQO136kOo0SHiLZi"
];

$body = json_encode([
    "number" => $number,
    "amount" => $amount,
    "method" => $method,
    "client" => $client,
    "details" => $details
]);

$ch = curl_init($payment_api_url);
curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
curl_setopt($ch, CURLOPT_POST, true);
curl_setopt($ch, CURLOPT_POSTFIELDS, $body);
$response = curl_exec($ch);

if (curl_errno($ch)) {
    http_response_code(500);
    echo json_encode(["message" => "Failed to connect to payment gateway: " . curl_error($ch)]);
    curl_close($ch);
    exit();
}

$response_data = json_decode($response, true);
if (!$response_data || $response_data['status'] != 200) {
    http_response_code($response_data['code'] ?? 500);
    echo json_encode(["message" => $response_data['message'] ?? "Failed to create payment"]);
    curl_close($ch);
    exit();
}

curl_close($ch);

// تحديث الجدول بالبيانات الجديدة
try {
    $stmt = $pdo->prepare("INSERT INTO transactions (user_id, amount, type, status, reference, client, details, created_at) VALUES (?, ?, ?, 'pending', ?, ?, ?, NOW())");
    $stmt->execute([
        $user_id,
        $amount,
        $method,
        $response_data['data']['reference'],
        $client,
        $details
    ]);

    echo json_encode(["message" => "Payment created successfully", "data" => $response_data['data']]);
} catch (PDOException $e) {
    http_response_code(500);
    echo json_encode(["message" => "Database error: " . $e->getMessage()]);
}
?>
