<?php
require '../config/db.php';
require '../utils/auth.php';

// الحصول على البيانات المرسلة في الطلب
$data = json_decode(file_get_contents("php://input"), true);

// التحقق من وجود المفاتيح المطلوبة
if (!isset($data['name'], $data['phone_number'], $data['password'])) {
    http_response_code(400);
    echo json_encode(["message" => "يجب كتابة الاسم ورقم الهاتف وكلمة المرور"]);
    exit();
}

$name = trim($data['name']);
$phone_number = trim($data['phone_number']);
$password = $data['password']; // كلمة المرور لا يتم تقليمها للحفاظ على الحروف

// التحقق من صحة الاسم: يسمح بالأحرف والأرقام والمسافات فقط
if (!preg_match("/^[a-zA-Z0-9\s]+$/", $name)) {
    http_response_code(400);
    echo json_encode(["message" => "تنسيق الاسم غير صالح"]);
    exit();
}

// التحقق من صحة رقم الهاتف: يجب أن يتكون من أرقام فقط وطول معين
if (!preg_match("/^\d{10,15}$/", $phone_number)) { // مثال: من 10 إلى 15 رقم
    http_response_code(400);
    echo json_encode(["message" => "تنسيق رقم الهاتف غير صحيح"]);
    exit();
}

// التحقق من قوة كلمة المرور: على سبيل المثال، يجب أن تكون 8 أحرف على الأقل
if (strlen($password) < 6) {
    http_response_code(400);
    echo json_encode(["message" => "يجب أن تكون كلمة المرور على الأقل 6 أحرف"]);
    exit();
}

// تنقية الاسم ورقم الهاتف لمنع XSS إذا تم عرضهما لاحقًا
$name = htmlspecialchars($name, ENT_QUOTES, 'UTF-8');
$phone_number = htmlspecialchars($phone_number, ENT_QUOTES, 'UTF-8');

try {
    // تشفير كلمة المرور
    $password_hash = hash_password($password);

    // إدخال المستخدم الجديد في قاعدة البيانات باستخدام استعلام محضر
    $stmt = $pdo->prepare("INSERT INTO users (name, phone_number, password_hash) VALUES (?, ?, ?)");
    $stmt->execute([$name, $phone_number, $password_hash]);

    // استجابة النجاح
    echo json_encode(["message" => "تم إنشاء الحساب بنجاح"]);
} catch (PDOException $e) {
    // التحقق إذا كان رقم الهاتف موجودًا مسبقًا
    if ($e->getCode() === '23000') { // كود الخطأ للقيود الفريدة
        http_response_code(400);
        echo json_encode(["message" => "رقم الهاتف هذا موجود من قبل"]);
    } else {
        http_response_code(500);
        echo json_encode(["message" => "يوجد خطأ أثناء تسجيل الحساب حاول بعد دقائق "]);
    }
}
?>
