<?php
// ملف: add_balance.php

require '../config/db.php'; // تأكد من مسار ملف إعدادات قاعدة البيانات

// تحديد نوع الاستجابة كـ JSON
header("Content-Type: application/json; charset=UTF-8");

// التحقق من طريقة الطلب
if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
    http_response_code(405); // طريقة غير مسموحة
    echo json_encode(["message" => "طريقة الطلب غير مسموحة. استخدم POST."]);
    exit();
}

// استخراج معرف المستخدم من بيانات الطلب (POST Parameters)
$data = json_decode(file_get_contents("php://input"), true);
$user_id = isset($data['user_id']) ? intval($data['user_id']) : null;

// التحقق من وجود معرف المستخدم
if (!$user_id) {
    http_response_code(400); // طلب غير صالح
    echo json_encode(["message" => "معرف المستخدم مطلوب"]);
    exit();
}

// التحقق من التوثيق (اختياري ولكن مستحسن)
// يمكنك إضافة رمز توثيق هنا للتحقق من صلاحية الطلب

try {
    // بدء معاملة قاعدة البيانات لضمان سلامة العمليات
    $pdo->beginTransaction();

    // جلب رصيد المستخدم مع قفل السجل لتجنب التعديلات المتزامنة
    $stmt = $pdo->prepare("SELECT balance FROM users WHERE id = ? FOR UPDATE");
    $stmt->execute([$user_id]);
    $user = $stmt->fetch(PDO::FETCH_ASSOC);

    if ($user) {
        $current_balance = floatval($user['balance']);

        // إضافة 1 جنيه إلى الرصيد
        $new_balance = $current_balance + 1.00;

        // تحديث الرصيد في قاعدة البيانات
        $update_stmt = $pdo->prepare("UPDATE users SET balance = ? WHERE id = ?");
        $update_stmt->execute([$new_balance, $user_id]);

        // تأكيد المعاملة
        $pdo->commit();

        // تنفيذ API آخر بعد الإضافة بنجاح (اختياري)
        // يمكنك استخدام cURL لتنفيذ طلب إلى API آخر إذا لزم الأمر
        /*
        $ch = curl_init("https://example.com/another_api_endpoint");
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode(["user_id" => $user_id, "added_amount" => 1.00]));
        curl_setopt($ch, CURLOPT_HTTPHEADER, ['Content-Type: application/json']);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($ch);
        $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        // يمكنك معالجة الاستجابة من API الآخر إذا لزم الأمر
        */

        // إرجاع استجابة نجاح
        http_response_code(200);
        echo json_encode([
            "message" => "تم إضافة 1 جنيه إلى رصيد المستخدم بنجاح",
            "new_balance" => number_format($new_balance, 2, '.', '')
        ]);
    } else {
        // المستخدم غير موجود
        $pdo->rollBack();
        http_response_code(404);
        echo json_encode(["message" => "المستخدم غير موجود"]);
    }
} catch (PDOException $e) {
    // التراجع عن المعاملة في حالة حدوث خطأ
    if ($pdo->inTransaction()) {
        $pdo->rollBack();
    }
    // التعامل مع أخطاء قاعدة البيانات
    error_log("PDOException: " . $e->getMessage());
    http_response_code(500);
    echo json_encode(["message" => "خطأ في قاعدة البيانات"]);
}
?>
