<?php
// ملف: get_user_balance.php

require '../config/db.php'; // تأكد من مسار ملف إعدادات قاعدة البيانات

// تحديد نوع الاستجابة كـ JSON
header("Content-Type: application/json; charset=UTF-8");

// التحقق من طريقة الطلب
if ($_SERVER['REQUEST_METHOD'] !== 'GET') {
    http_response_code(405); // طريقة غير مسموحة
    echo json_encode(["message" => "طريقة الطلب غير مسموحة. استخدم GET."]);
    exit();
}

// استخراج معرف المستخدم من معلمات الاستعلام (Query Parameters)
$user_id = isset($_GET['user_id']) ? intval($_GET['user_id']) : null;

// التحقق من وجود معرف المستخدم
if (!$user_id) {
    http_response_code(400); // طلب غير صالح
    echo json_encode(["message" => "معرف المستخدم مطلوب"]);
    exit();
}

// التحقق من التوثيق (اختياري ولكن مستحسن)
// يمكنك إضافة رمز توثيق هنا للتحقق من صلاحية الطلب

try {
    // إعداد استعلام لجلب رصيد المستخدم
    $stmt = $pdo->prepare("SELECT balance FROM users WHERE id = ?");
    $stmt->execute([$user_id]);
    $user = $stmt->fetch(PDO::FETCH_ASSOC);

    if ($user) {
        // إرجاع رصيد المستخدم
        http_response_code(200);
        echo json_encode([
            "message" => "تم جلب رصيد المستخدم بنجاح",
            "balance" => $user['balance']
        ]);
    } else {
        // المستخدم غير موجود
        http_response_code(404);
        echo json_encode(["message" => "المستخدم غير موجود"]);
    }
} catch (PDOException $e) {
    // التعامل مع أخطاء قاعدة البيانات
    error_log("PDOException: " . $e->getMessage());
    http_response_code(500);
    echo json_encode(["message" => "خطأ في قاعدة البيانات"]);
}
?>
